using System;
using System.Collections.Generic;
using System.Text;
using Brume;

namespace BrumeParticles.Particles
{
    /// <summary>
    /// Represents the particles Emission Mode
    /// </summary>
    public enum eParticleEmissionMode
    {
        /// <summary>
        /// Continiously create particles
        /// </summary>
        Continious,
        /// <summary>
        /// Create particles only once
        /// </summary>
        Once,
        /// <summary>
        /// Manage the emission manually
        /// </summary>
        Manual
    }




    /// <summary>
    /// Interface for particles Emitter
    /// </summary>
    public interface IParticleEmitter
    {
        // Methods
        /// <summary>
        /// Generates the direction.
        /// </summary>
        /// <returns></returns>
        BrumeVector GenerateDirection();
        /// <summary>
        /// Generates the position.
        /// </summary>
        /// <returns></returns>
        BrumeVector GeneratePosition();
        /// <summary>
        /// Gets the speed.
        /// </summary>
        /// <returns></returns>
        float GetSpeed();

        // Properties

        /// <summary>
        /// Gets or sets the center.
        /// </summary>
        /// <value>The center.</value>
        BrumeVector Center { get; set; }

        /// <summary>
        /// Gets or sets the emission force.
        /// </summary>
        /// <value>The emission force.</value>
        float EmissionForce { get; set; }

        /// <summary>
        /// Gets or sets the emitter rate.
        /// </summary>
        /// <value>The emitter rate.</value>
        float EmitterRate { get; set; }

        /// <summary>
        /// Gets or sets the emission mode.
        /// </summary>
        /// <value>The mode.</value>
        eParticleEmissionMode Mode { get; set; }
    }


}
