using System;
using System.Collections.Generic;
using System.Text;
using Brume;

namespace BrumeParticles.Particles
{
    /// <summary>
    /// Emittes a particle from a point
    /// </summary>
    public class ParticlePointEmitter : IParticleEmitter
    {

        private float _DirectionNoise;
        private float _EmissionForce;
        private eParticleEmissionMode _EmissionMode;
        private float _ForceNoise;
        private BrumeObject _Node;
        private float _PositionNoise;
        private float _Rate;
        public BrumeVector EmissionDirection;
        public BrumeVector _position;


        /// <summary>
        /// Gets or sets the center.
        /// </summary>
        /// <value>The center.</value>
        public BrumeVector Center
        {
            get
            {
                if (_Node == null)
                    return _position;
                else
                    return this._Node.Pos;
            }
            set
            {
                if (_Node == null)
                    this._position = value;
                else
                    this._Node.Pos = value;

            }
        }

        /// <summary>
        /// Gets or sets the direction noise.
        /// </summary>
        /// <value>The direction noise.</value>
        public float DirectionNoise
        {
            get
            {
                return this._DirectionNoise;
            }
            set
            {
                this._DirectionNoise = value;
            }
        }

        /// <summary>
        /// Gets or sets the emission force.
        /// </summary>
        /// <value>The emission force.</value>
        public float EmissionForce
        {
            get
            {
                return this._EmissionForce;
            }
            set
            {
                this._EmissionForce = value;
            }
        }

        /// <summary>
        /// Gets or sets the emitter rate.
        /// </summary>
        /// <value>The emitter rate.</value>
        public float EmitterRate
        {
            get
            {
                return this._Rate;
            }
            set
            {
                this._Rate = value;
            }
        }

        /// <summary>
        /// Gets or sets the force noise.
        /// </summary>
        /// <value>The force noise.</value>
        public float ForceNoise
        {
            get
            {
                return this._ForceNoise;
            }
            set
            {
                this._ForceNoise = value;
            }
        }

        /// <summary>
        /// Gets or sets the emission mode.
        /// </summary>
        /// <value>The mode.</value>
        public eParticleEmissionMode Mode
        {
            get
            {
                return this._EmissionMode;
            }
            set
            {
                this._EmissionMode = value;
            }
        }

        /// <summary>
        /// Gets the object.
        /// </summary>
        /// <value>The object.</value>
        public BrumeObject Object
        {
            get
            {
                return this._Node;
            }
        }


        /// <summary>
        /// Gets or sets the position noise.
        /// </summary>
        /// <value>The position noise.</value>
        public float PositionNoise
        {
            get
            {
                return this._PositionNoise;
            }
            set
            {
                this._PositionNoise = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParticlePointEmitter"/> class.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="EmissionMode">The emission mode.</param>
        /// <param name="EmitterPosition">The emitter position.</param>
        /// <param name="EmissionDirection">The emission direction.</param>
        /// <param name="EmissionForce">The emission force.</param>
        /// <param name="Rate">The rate.</param>
        public ParticlePointEmitter(BrumeObject node, eParticleEmissionMode EmissionMode, BrumeVector EmitterPosition, BrumeVector EmissionDirection, float EmissionForce, float Rate)
        {
            this._position = EmitterPosition;
            this._EmissionMode = EmissionMode;
            this._Rate = Rate;
            this.EmissionDirection = EmissionDirection;
            this.EmissionForce = EmissionForce;
            this._PositionNoise = 0f;
            this._DirectionNoise = 0f;

            this._Node = node;

        }

        /// <summary>
        /// Generates the direction.
        /// </summary>
        /// <returns></returns>
        public BrumeVector GenerateDirection()
        {
            return (this.EmissionDirection + Randomizer.GetRandomVector3(-this._DirectionNoise, this._DirectionNoise));
        }

        /// <summary>
        /// Generates the position.
        /// </summary>
        /// <returns></returns>
        public BrumeVector GeneratePosition()
        {
            return (Center + Randomizer.GetRandomVector3(-this._PositionNoise, this._PositionNoise));
        }

        /// <summary>
        /// Gets the speed.
        /// </summary>
        /// <returns></returns>
        public float GetSpeed()
        {
            return this._EmissionForce; //(this._EmissionForce + Randomizer.GetRandomFloat(-this._EmissionForce, this._EmissionForce));
        }




    }


}
