using System;
using System.Collections.Generic;
using System.Text;
using Brume;
using System.Drawing;

namespace BrumeParticles.Particles
{
    /// <summary>
    /// Represents a particle
    /// </summary>
    public class Particle
    {

        /// <summary>
        /// A color that can be negative
        /// </summary>
        public struct deltaColor
        {
            public deltaColor(int red , int green , int blue)
            {
                Red = red;
                Blue = blue;
                Green = green;
            }


            public int Red;
            public int Blue;
            public int Green;
        }

        // COLOR
        public float Alpha;
        public Color Color;
        internal deltaColor ColorDelta;
        internal Color InitialColor;

        //SCALE
        public float Scale;
        internal float ScaleDelta;

        // MOVE
        public BrumeVector Position;
        public BrumeVector Direction;
        public float Speed;
        internal float Acceleration;


        // LIFETIME
        public float CurrentLifeTime;
        public float LifeTime;
        public float NormalizedLifeTime;


        /// <summary>
        /// List of processors
        /// </summary>
        internal FastList<eParticleProcessorState> ProcessorsState = new FastList<eParticleProcessorState>();




        /// <summary>
        /// Initializes a new instance of the <see cref="Particle"/> class.
        /// </summary>
        /// <param name="Position">The position.</param>
        /// <param name="Direction">The direction.</param>
        /// <param name="Color">The color.</param>
        /// <param name="Scale">The scale.</param>
        /// <param name="Alpha">The alpha.</param>
        /// <param name="LifeTime">The life time.</param>
        /// <param name="Speed">The speed.</param>
        public Particle(BrumeVector Position, BrumeVector Direction, Color Color, float Scale, float Alpha, float LifeTime, float Speed)
        {
            this.Reset(Position, Direction, Color, Scale, Alpha, LifeTime, Speed);
        }

        /// <summary>
        /// Resets the specified particle.
        /// </summary>
        /// <param name="Position">The position.</param>
        /// <param name="Direction">The direction.</param>
        /// <param name="Color">The color.</param>
        /// <param name="Scale">The scale.</param>
        /// <param name="Alpha">The alpha.</param>
        /// <param name="LifeTime">The life time.</param>
        /// <param name="Speed">The speed.</param>
        public void Reset(BrumeVector Position, BrumeVector Direction, Color Color, float Scale, float Alpha, float LifeTime, float Speed)
        {
            this.Position = Position;
            this.Direction = Direction;
            this.Color = this.InitialColor = Color;
            this.Scale = Scale;
            this.Alpha = Alpha;
            this.LifeTime = LifeTime;
            this.CurrentLifeTime = 0f;
            this.Speed = Speed;
            this.Acceleration = 0f;
            this.ScaleDelta = 0f;
        }

        /// <summary>
        /// Resets the processors.
        /// </summary>
        public void ResetProcessors()
        {
            for (int i = 0; i < this.ProcessorsState.Count; i++)
            {
                this.ProcessorsState.Set(i, eParticleProcessorState.Dead);
            }
        }

        /// <summary>
        /// Updates the particle
        /// </summary>
        /// <param name="DeltaTime">The delta time.</param>
        /// <returns></returns>
        public bool Update(float DeltaTime)
        {
            this.CurrentLifeTime += DeltaTime;
            this.NormalizedLifeTime = this.CurrentLifeTime / this.LifeTime;
            this.Position += (BrumeVector)(this.Direction * (this.Speed * DeltaTime));
            return (this.CurrentLifeTime < this.LifeTime);
        }

    }


}
