using System;
using System.Collections.Generic;
using System.Text;

namespace BrumeParticles.Particles
{
    /// <summary>
    /// Interface for the particle processors
    /// </summary>
    public interface IParticleProcessor
    {
  
        /// <summary>
        /// Checks the state.
        /// </summary>
        /// <param name="ParticleNormalizedTime">The particle normalized time.</param>
        /// <param name="LastState">State of the last.</param>
        /// <returns></returns>
        eParticleProcessorState CheckState(float ParticleNormalizedTime, eParticleProcessorState LastState);
        
        /// <summary>
        /// Initializes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        void Initialize(Particle Particle);

        /// <summary>
        /// Processes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        /// <param name="DeltaTime">The delta time.</param>
        void Process(Particle Particle, float DeltaTime);

        /// <summary>
        /// Gets the state.
        /// </summary>
        /// <value>The state.</value>
        eParticleProcessorState State { get; }
    }

 

}
