using System;
using System.Collections.Generic;
using System.Text;

namespace BrumeParticles.Particles
{
    public class ParticleAcceleratorProcessor : ParticleProcessor
    {
        /// <summary>
        /// Acceleration of the particles
        /// </summary>
        public float Acceleration;

        /// <summary>
        /// Acceleration Noise 
        /// </summary>
        public float AccelerationNoise;


        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleAcceleratorProcessor"/> class.
        /// </summary>
        /// <param name="Acceleration">The acceleration.</param>
        /// <param name="AccelerationNoise">The acceleration noise.</param>
        public ParticleAcceleratorProcessor(float Acceleration, float AccelerationNoise)
            : this(Acceleration, AccelerationNoise, ParticleProcessorTimer.AlwaysActiveTimer)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleAcceleratorProcessor"/> class.
        /// </summary>
        /// <param name="Acceleration">The acceleration.</param>
        /// <param name="AccelerationNoise">The acceleration noise.</param>
        /// <param name="Timer">The timer.</param>
        public ParticleAcceleratorProcessor(float Acceleration, float AccelerationNoise, ParticleProcessorTimer Timer)
            : base(Timer)
        {
            this.Acceleration = Acceleration;
            this.AccelerationNoise = AccelerationNoise;
        }

        /// <summary>
        /// Initializes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        public sealed override void Initialize(Particle Particle)
        {
            Particle.Acceleration = this.Acceleration + Randomizer.GetRandomFloat(-this.AccelerationNoise, this.AccelerationNoise);
        }

        /// <summary>
        /// Processes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        /// <param name="DeltaTime">The delta time.</param>
        public sealed override void Process(Particle Particle, float DeltaTime)
        {
            Particle.Speed += Particle.Acceleration * DeltaTime;
        }
    }


}
