using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace BrumeParticles.Particles
{
    public class ParticleColorProcessor : ParticleProcessor
    {



        /// <summary>
        /// Wantes color at the end of the process
        /// </summary>
        public Color ColorEnd;


        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleColorProcessor"/> class.
        /// </summary>
        /// <param name="ColorEnd">The color end.</param>
        public ParticleColorProcessor(Color ColorEnd)
            : this(ColorEnd, ParticleProcessorTimer.AlwaysActiveTimer)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleColorProcessor"/> class.
        /// </summary>
        /// <param name="ColorEnd">The color end.</param>
        /// <param name="Timer">The timer.</param>
        public ParticleColorProcessor(Color ColorEnd, ParticleProcessorTimer Timer)
            : base(Timer)
        {
            this.ColorEnd = ColorEnd;
        }

        /// <summary>
        /// Initializes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        public sealed override void Initialize(Particle Particle)
        {
            Particle.InitialColor = Particle.Color;
            Particle.ColorDelta = RemoveColor( ColorEnd , Particle.Color );
        }

        /// <summary>
        /// Processes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        /// <param name="DeltaTime">The delta time.</param>
        public sealed override void Process(Particle Particle, float DeltaTime)
        {
            float num = (Particle.NormalizedLifeTime - this.Timer.StartAt) / (this.Timer.EndAt - this.Timer.StartAt);
            Particle.deltaColor color = ( MultiplyColor ( Particle.ColorDelta , num));
            float a = Particle.Color.A;
            Particle.Color = AddColor( Particle.InitialColor , color );
        }

        /// <summary>
        /// Removes the color.
        /// </summary>
        /// <param name="color1">The color1.</param>
        /// <param name="color2">The color2.</param>
        /// <returns></returns>
        private static Particle.deltaColor RemoveColor(Color color1, Color color2)
        {
            int red = color1.R - color2.R;
            int green = color1.G - color2.G;
            int blue = color1.B - color2.B;
           

            return new Particle.deltaColor(red,green,blue);
        }

        /// <summary>
        /// Adds the color.
        /// </summary>
        /// <param name="color1">The color1.</param>
        /// <param name="color2">The color2.</param>
        /// <returns></returns>
        private static Color AddColor(Color color1, Particle.deltaColor color2)
        {
            int red = color1.R + color2.Red;
            int green = color1.G + color2.Green;
            int blue = color1.B + color2.Blue;

            if (red > 255)
                red = 255;
            if (green > 255)
                green = 255;
            if (blue > 255)
                blue = 255;

            return Color.FromArgb(red, green, blue);

        }

        /// <summary>
        /// Multiplies the color.
        /// </summary>
        /// <param name="color1">The color1.</param>
        /// <param name="k">The k.</param>
        /// <returns></returns>
        private static Particle.deltaColor MultiplyColor(Particle.deltaColor color1, float k)
        {
            return new Particle.deltaColor((int)(color1.Red * k), (int)(color1.Green * k), (int)(color1.Blue * k));

        }

    }

 

}
