using System;
using System.Collections.Generic;
using System.Text;
using Brume;

namespace BrumeParticles.Particles
{
    class ParticleConcentrationProcessor : ParticleProcessor
    {
        // The emmitter of particle
        IParticleEmitter ParticleEmitter;


        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleConcentrationProcessor"/> class.
        /// </summary>
        /// <param name="ParticleEmitter">The particle emitter.</param>
        public ParticleConcentrationProcessor(IParticleEmitter ParticleEmitter)
            : this(ParticleEmitter, ParticleProcessorTimer.AlwaysActiveTimer)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleConcentrationProcessor"/> class.
        /// </summary>
        /// <param name="ParticleEmitter">The particle emitter.</param>
        /// <param name="Timer">The timer.</param>
        public ParticleConcentrationProcessor(IParticleEmitter ParticleEmitter, ParticleProcessorTimer Timer)
            : base(Timer)
        {
            this.ParticleEmitter = ParticleEmitter;
        }

        /// <summary>
        /// Initializes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        public sealed override void Initialize(Particle Particle)
        {
            Particle.Direction = ParticleEmitter.Center - Particle.Position;
            Particle.Direction.Normalize();

        }

        /// <summary>
        /// Processes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        /// <param name="DeltaTime">The delta time.</param>
        public sealed override void Process(Particle Particle, float DeltaTime)
        {
            BrumeVector newDirection = ParticleEmitter.Center - Particle.Position;
            newDirection.Normalize();

            float length = (Particle.Direction - newDirection).Length();

            // Bad way to do it : would be better to change lifetime at beginning
            // destroy the particle
            if (length > 0.005f)
                Particle.CurrentLifeTime = Particle.LifeTime;

        }
    }
}
