using System;
using System.Collections.Generic;
using System.Text;

namespace BrumeParticles.Particles
{
    /// <summary>
    /// base class for particle processor
    /// </summary>
    public abstract class ParticleProcessor : IParticleProcessor
    {
        
        private eParticleProcessorState _State;
        public ParticleProcessorTimer Timer;

        /// <summary>
        /// Gets the state.
        /// </summary>
        /// <value>The state.</value>
        public eParticleProcessorState State
        {
            get
            {
                return this._State;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleProcessor"/> class.
        /// </summary>
        public ParticleProcessor()
            : this(ParticleProcessorTimer.AlwaysActiveTimer)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleProcessor"/> class.
        /// </summary>
        /// <param name="Timer">The timer.</param>
        public ParticleProcessor(ParticleProcessorTimer Timer)
        {
            this.Timer = Timer;
        }

        /// <summary>
        /// Checks the state.
        /// </summary>
        /// <param name="ParticleNormalizedTime">The particle normalized time.</param>
        /// <param name="LastState">State of the last.</param>
        /// <returns></returns>
        public eParticleProcessorState CheckState(float ParticleNormalizedTime, eParticleProcessorState LastState)
        {
            bool flag = true;
            if (!this.Timer.AlwaysActive && ((ParticleNormalizedTime < this.Timer.StartAt) || (ParticleNormalizedTime > this.Timer.EndAt)))
            {
                flag = false;
            }
            if (flag && (LastState == eParticleProcessorState.Dead))
            {
                this._State = eParticleProcessorState.Alive;
                return eParticleProcessorState.JustAlive;
            }
            if (!flag && (LastState == eParticleProcessorState.Alive))
            {
                this._State = eParticleProcessorState.Dead;
                return eParticleProcessorState.JustDead;
            }
            this._State = flag ? eParticleProcessorState.Alive : eParticleProcessorState.Dead;
            return this._State;
        }

        public abstract void Initialize(Particle Particle);
        public abstract void Process(Particle Particle, float DeltaTime);


        
    }

 

}
