using System;
using System.Collections.Generic;
using System.Text;

namespace BrumeParticles.Particles
{
    public class ParticleScaleProcessor : ParticleProcessor
    {

        /// <summary>
        /// Scale at the end of the process
        /// </summary>
        public float ScaleEnd;


        /// <summary>
        /// Noise of the scale
        /// </summary>
        public float ScaleNoise;


        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleScaleProcessor"/> class.
        /// </summary>
        /// <param name="ScaleEnd">The scale end.</param>
        /// <param name="ScaleNoise">The scale noise.</param>
        public ParticleScaleProcessor(float ScaleEnd, float ScaleNoise)
            : this(ScaleEnd, ScaleNoise, ParticleProcessorTimer.AlwaysActiveTimer)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParticleScaleProcessor"/> class.
        /// </summary>
        /// <param name="ScaleEnd">The scale end.</param>
        /// <param name="ScaleNoise">The scale noise.</param>
        /// <param name="Timer">The timer.</param>
        public ParticleScaleProcessor(float ScaleEnd, float ScaleNoise, ParticleProcessorTimer Timer)
            : base(Timer)
        {
            this.ScaleEnd = ScaleEnd;
            this.ScaleNoise = ScaleNoise;
        }

        /// <summary>
        /// Initializes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        public sealed override void Initialize(Particle Particle)
        {
            Particle.ScaleDelta = (this.ScaleEnd + Randomizer.GetRandomFloat(-this.ScaleNoise, this.ScaleNoise)) - Particle.Scale;
        }

        /// <summary>
        /// Processes the specified particle.
        /// </summary>
        /// <param name="Particle">The particle.</param>
        /// <param name="DeltaTime">The delta time.</param>
        public sealed override void Process(Particle Particle, float DeltaTime)
        {
            Particle.Scale += Particle.ScaleDelta * DeltaTime / 1000;
        }
    }


}
