using System;
using System.Collections.Generic;
using System.Text;
using Brume;
using System.Drawing;
using BrumeParticles.Particles;

namespace BrumeParticles
{
    class ParticlesSample : Brume.Brume
    {
        public ParticlesSample()
            : base()
        {
            //this.InitStates.Mode = BrumeInitParams.MODE_FULLSCREEN;

            BrumeFileManager.AddDirForAll("Data/Internal");
        }

        override protected void GameInit()
        {
            // camera setup
            BrumeVector cameraPos = new BrumeVector(0.0f, 0.0f, -20.0f);
            BrumeVector cameraDir = new BrumeVector(0, 0.0f, 1f);
            BrumeVector cameraUp = new BrumeVector(0.0f, 1.0f, 0.0f);

            BrumeFPSCamera camera = new BrumeFPSCamera(this, "camera", cameraPos, cameraDir, cameraUp);

            this.SetActiveCamera(camera);

            // buffer clear color setup
            this.BufferClearColor = Color.Black;


            ColdEffect();
            FireEffect();
            PlasmaEffect();

        }

        private void PlasmaEffect()
        {

            ParticlePointEmitter PointEmitter3 = new ParticlePointEmitter(null, eParticleEmissionMode.Continious, new BrumeVector(-2, 0, 0.0f),
                new BrumeVector(0, 1, 0), 0.00000007f, 0.001f);
            PointEmitter3.PositionNoise = 0.5f;
            PointEmitter3.DirectionNoise = 2f;
            


            ParticleSystem ParticuleSystem3 = new ParticleSystem(this, GetTexture("particle.tga"), 1, 30,
            1000, 500, Color.FromArgb(25, 50, 170), PointEmitter3);
            ParticuleSystem3.Scale = 0.8f;
            ParticuleSystem3.ColorNoise = Color.FromArgb(10, 30, 80);

            ParticleScaleProcessor scaleProcessor = new ParticleScaleProcessor(2f, 0.5f);

            ParticuleSystem3.AddProcessor(scaleProcessor);


        }


        private void FireEffect()
        {
            ParticlePointEmitter PointEmitter3 = new ParticlePointEmitter(null, eParticleEmissionMode.Continious, new BrumeVector(2.0f, 0f, 0.0f),
               new BrumeVector(0,1,0), 0.00001f, 0.03f);

            PointEmitter3.DirectionNoise = 0.2f;
            PointEmitter3.PositionNoise = 0.5f;

            ParticleSystem ParticuleSystem3 = new ParticleSystem(this ,GetTexture("particle.tga"), 1, 80,
    1000.0f, 400,Color.FromArgb(130,70,0), PointEmitter3);


            ParticleAcceleratorProcessor AcceleratorProcessor = new ParticleAcceleratorProcessor(0.00002f, 0.000005f);

            ParticleScaleProcessor ScaleProcessor2 = new ParticleScaleProcessor(2.0f, 0.2f);

            ParticleColorProcessor ColorProcessor = new ParticleColorProcessor(Color.FromArgb(50,50,50), new ParticleProcessorTimer(0.5f, 1.0f));



            ParticuleSystem3.AddProcessor(AcceleratorProcessor);
            ParticuleSystem3.AddProcessor(ScaleProcessor2);
            ParticuleSystem3.AddProcessor(ColorProcessor);
        }


        private void ColdEffect()
        {
            ParticlePointEmitter PointEmitter3 = new ParticlePointEmitter(null, eParticleEmissionMode.Continious, new BrumeVector(),
            new BrumeVector(0.0f, 1f, 0.0f), 0.0007f, 0.01f);


            PointEmitter3.PositionNoise = 1f;

            ParticleSystem _particleSystem = new ParticleSystem(this, GetTexture("starsnow.jpg"), 1, 60,
                1500, 1000, Color.FromArgb(100, 150, 200), PointEmitter3);
            _particleSystem.Scale = 0.001f;

            ParticleScaleProcessor scale = new ParticleScaleProcessor(0.7f, 0.1f,new ParticleProcessorTimer(0.1f,1));
            ParticleConcentrationProcessor concentration = new ParticleConcentrationProcessor(PointEmitter3);

            _particleSystem.AddProcessor(scale);
 
            _particleSystem.AddProcessor(concentration);

        }

        private float moveSpeed = 100.0f;

        protected override void MoveScene(float fElapsedTime)
        {
            // retreives active camera
            BrumeFPSCamera camera = (BrumeFPSCamera)this.ActiveCamera;

            // if camera present and not in full console mode
            if (camera != null && this.DebugConsole.debugLevel != 2)
            {
                // orient
                camera.FPSOrient((float)this.MouseDeltaX / (float)2, (float)this.MouseDeltaY / (float)2);

                // move speed
                if (MouseDeltaZ > 0.0f)
                    moveSpeed *= 0.5f;
                else
                    if (MouseDeltaZ < 0.0f)
                        moveSpeed /= 0.5f;

                // move
                if (BrumeKeyboard.IsPressed(BrumeKey.W))
                    camera.FPSMove(camera.Dir * (fElapsedTime / moveSpeed));
                if (BrumeKeyboard.IsPressed(BrumeKey.S))
                    camera.FPSMove(-camera.Dir * (fElapsedTime / moveSpeed));
                if (BrumeKeyboard.IsPressed(BrumeKey.D))
                    camera.FPSMove(camera.Right * (fElapsedTime / moveSpeed));
                if (BrumeKeyboard.IsPressed(BrumeKey.A))
                    camera.FPSMove(-camera.Right * (fElapsedTime / moveSpeed));

            }


            // brume call
            base.MoveScene(fElapsedTime);
        }


        protected override void AfterRendering()
        {
            base.AfterRendering();
            // exit ?
            if (BrumeKeyboard.IsPressed(BrumeKey.Escape))
            {
                this.Close();
                return;
            }
        }

        public static void Main()
        {
            using (ParticlesSample game = new ParticlesSample())
            {
                game.Play();
            }
        }
    }
}
