using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace BrumeParticles.Particles
{
    /// <summary>
    /// Like a List<> but faster to use
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class FastList<T>
    {
        // Fields
        internal T[] Elements;
        internal int InternalPointer;

        // Methods
        public FastList()
        {
            this.Elements = new T[2];
        }

        public FastList(int ElementsCount)
        {
            this.Elements = new T[ElementsCount];
        }

        public void AddElement(T Data)
        {
            if (this.Elements.Length <= this.InternalPointer)
            {
                this.Resize(this.Elements.Length * 2);
            }
            this.Elements[this.InternalPointer] = Data;
            this.InternalPointer++;
        }

        public void FastClear()
        {
            this.InternalPointer = 0;
        }

        public IEnumerator GetEnumerator()
        {
            return this.Elements.GetEnumerator();
        }

        public void Pop()
        {
            if (this.InternalPointer > 0)
            {
                this.InternalPointer--;
            }
        }

        public void Pop(int Count)
        {
            if ((this.InternalPointer - Count) > 0)
            {
                this.InternalPointer -= Count;
            }
        }

        public void Resize(int NewSize)
        {
            T[] destinationArray = new T[NewSize];
            Array.Copy(this.Elements, destinationArray, this.InternalPointer);
            this.Elements = destinationArray;
        }

        public void Set(int Index, T Value)
        {
            this.Elements[Index] = Value;
        }

        // Properties
        public int Count
        {
            get
            {
                return this.InternalPointer;
            }
        }

        public T[] GetElements
        {
            get
            {
                T[] destinationArray = new T[this.InternalPointer];
                Array.Copy(this.Elements, destinationArray, this.InternalPointer);
                return destinationArray;
            }
        }

        public T this[int Index]
        {
            get
            {
                return this.Elements[Index];
            }
        }

        public int Pointer
        {
            get
            {
                return (this.InternalPointer - 1);
            }
        }
    }


}
