using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Brume;

namespace BrumeParticles.Particles
{
    /// <summary>
    /// Generate Random vector and color
    /// </summary>
    public static class Randomizer
    {

        /// <summary>
        /// 
        /// </summary>
        public static Random globalRandomGenerator = GenerateNewRandomGenerator();


        /// <summary>
        /// Generates the new random generator.
        /// </summary>
        /// <returns></returns>
        public static Random GenerateNewRandomGenerator()
        {
            globalRandomGenerator = new Random(0);
            return globalRandomGenerator;
        }


        /// <summary>
        /// Gets the random byte.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="max">The max.</param>
        /// <returns></returns>
        public static byte GetRandomByte(byte min, byte max)
        {
            return (byte)globalRandomGenerator.Next(min, max);
        }

        /// <summary>
        /// Gets the random color.
        /// </summary>
        /// <returns></returns>
        public static Color GetRandomColor()
        {
            return GetRandomColor(Color.Black, Color.White);
        }

        /// <summary>
        /// Gets the random color.
        /// </summary>
        /// <param name="MinColor">Color of the min.</param>
        /// <param name="MaxColor">Color of the max.</param>
        /// <returns></returns>
        public static Color GetRandomColor(Color MinColor, Color MaxColor)
        {
            return Color.FromArgb((int)GetRandomFloat(MinColor.R, MaxColor.R), (int)GetRandomFloat(MinColor.G, MaxColor.G), (int)GetRandomFloat(MinColor.B, MaxColor.B));
        }

        /// <summary>
        /// Gets the random float.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="max">The max.</param>
        /// <returns></returns>
        public static float GetRandomFloat(float min, float max)
        {
            return ((((float)globalRandomGenerator.NextDouble()) * (max - min)) + min);
        }

        /// <summary>
        /// Gets the random int.
        /// </summary>
        /// <param name="max">The max.</param>
        /// <returns></returns>
        public static int GetRandomInt(int max)
        {
            return globalRandomGenerator.Next(max);
        }


        /// <summary>
        /// Gets the random vector3.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="max">The max.</param>
        /// <returns></returns>
        public static BrumeVector GetRandomVector3(float min, float max)
        {
            return new BrumeVector(GetRandomFloat(min, max), GetRandomFloat(min, max), GetRandomFloat(min, max));
        }

        /// <summary>
        /// Gets the random vector3.
        /// </summary>
        /// <param name="Center">The center.</param>
        /// <param name="MinRadius">The min radius.</param>
        /// <param name="MaxRadius">The max radius.</param>
        /// <param name="HorizontalDelta">The horizontal delta.</param>
        /// <param name="VerticalDelta">The vertical delta.</param>
        /// <returns></returns>
        public static BrumeVector GetRandomVector3(BrumeVector Center, float MinRadius, float MaxRadius, float HorizontalDelta, float VerticalDelta)
        {
            float randomFloat = GetRandomFloat(MinRadius, MaxRadius);
            float angle = GetRandomFloat(-HorizontalDelta, HorizontalDelta);
            float num3 = GetRandomFloat(-VerticalDelta, VerticalDelta);
            BrumeVector vector = new BrumeVector();
            vector.X = ((float)Math.Cos(num3) * (float)Math.Cos(angle)) * randomFloat;
            vector.Y = (float)Math.Sin(num3) * randomFloat;
            vector.Z = ((float)Math.Cos(num3) * (float)Math.Sin(angle)) * randomFloat;
            return (vector + Center);
        }





    }


}
