float4x4 matWorldViewProj;          // World * View * Projection matrix
float4x4 matInvertTransposeWorld;   // transpose invert world
float3 lightDir;                    // light direction
float3 amb;                         // ambient intensity
float3 diff;                        // diffuse intensity


struct VS_OUTPUT 
{
    float4 Pos   : POSITION;   
    float3 Light : TEXCOORD0;
    float3 Norm : TEXCOORD1;
};


VS_OUTPUT DiffuseVS( float4 vPos : POSITION, float3 Norm : NORMAL)
{
    VS_OUTPUT Output;
    
    Output.Pos = mul(vPos, matWorldViewProj);
    Output.Light = normalize(lightDir);
    Output.Norm = normalize(mul(matInvertTransposeWorld, Norm));

    return Output;    
}


struct PS_OUTPUT
{
    float4 Col : COLOR0;  
};


PS_OUTPUT DiffusePS(float3 Light : TEXCOORD0, float3 Norm : TEXCOORD1) 
{ 
    PS_OUTPUT Output;
    
    // Lambert
    Output.Col = float4(0,0,0,0);
    Output.Col.xyz = amb + diff * saturate(dot(Norm, Light));
	
    return Output;
}



technique DiffuseShader
{
    pass P0
    {          
        VertexShader = compile vs_1_1 DiffuseVS( );
        PixelShader  = compile ps_1_4 DiffusePS( );
    }

}
